<?php



// Create main window

class MainWindow
{  protected $wbEv;   
   protected $windowTitle;
   protected $window;
   protected $menu, $menuItem, $console, $bankFrame;
   protected $infoText;
  
   function __construct($windowTitle, $infoText)
   {  
      $this->wbEv =& $GLOBALS['wbEvents'];
      $this->windowTitle = $windowTitle;  
      $this->infoText = $infoText;
      $this->createWindow();       
   }
   
    
   function createWindow()
   {  
      $wbEv =& $this->wbEv;
      $this->console = false;
      
      $metatileEditor = new MetatileEditorWindow('Background Tile Editor');
      $wbEv->metatileEditor =& $metatileEditor;
      
      $marioEditor = new MarioEditorWindow('Mario Sprite Editor');   
      $wbEv->marioEditor =& $marioEditor;
      
      $enemyEditor = new EnemyEditorWindow('Enemy Sprite Editor');   
      $wbEv->enemyEditor =& $enemyEditor;
      
      $colorEditor = new colorEditorWindow('Color Editor');
      $wbEv->colorEditor =& $colorEditor;      
      
      $wbEv->createWindowEvent(array($this,'windowHandler'),
        $this->window = wb_create_window( 
            NULL, ResizableWindow  , $this->windowTitle, WBC_CENTER, WBC_CENTER, 500, 250, 
               WBC_NOTIFY,  WBC_RESIZE ));
     
      //$wbEv->mainWindow =& $this->window;
      
      $menuItem = array();      
      // Add Menu   
      $this->menu = wb_create_control($this->window, Menu, 
         array
         (  "&File",
      		   array($wbEv->createEvent(array($this,'openrom')),	"&Open ROM..."),
      		   array($menuItem[] = $wbEv->createEvent(array($this,'saverom')),	"Save &As..."),
      		   null,
      		   array(IDCLOSE,		"E&xit\tAlt+F4"),
      		"&Edit",		   
      		   array($menuItem[] = $wbEv->createEvent(array($metatileEditor,'showEditor')),	"&Background metatiles..."),
      		   array($menuItem[] = $wbEv->createEvent(array($marioEditor,'showEditor')),	"&Mario Sprites..."),
      		   array($menuItem[] = $wbEv->createEvent(array($enemyEditor,'showEditor')),	"&Enemy Sprites..."),
      		   array($menuItem[] = $wbEv->createEvent(array($colorEditor,'showEditor')),	"&Palettes..."),
      	));
      
      foreach ($menuItem as $n=>$v)
      {  wb_set_enabled(wb_get_control($this->menu, $v), false);
      }
      $this->menuItem =& $menuItem;
      // Add controls  
      $this->console = wb_create_control($this->window, 
         EditBox, '', 10, 10,470, 140, 0, WBC_MULTILINE | WBC_READONLY, 0, 0);
      
      
      $wbEv->console =& $this->console;
      $this->windowSize = $this->getSize();
      $this->sizeConsole();
      $this->timer = wb_create_timer($this->window, $wbEv->createEvent(array($this,'checkSize')),100);
      
      $this->toConsole($this->infoText, false);
      
      
   }

   
   function windowHandler($params)
   {  if ($params['id'] == IDCLOSE)
      {  $this->closeWindow();
         return true;
      } 
      
      if ($params['param1'] == WBC_RESIZE)
      {  $this->sizeConsole();         
      }
      
      
   }
   
   function closeWindow()
   {              
         wb_destroy_window($this->window);         
         exit;
   }   
   
   function openrom($params)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;
      
      $filename = wb_sys_dlg_open($this->window, "Open ROM...", $wbEv->file_filter);
            
      
      if ($filename)
      {  foreach ($this->menuItem as $n=>$v)
         {  wb_set_enabled(wb_get_control($this->menu, $v), false);
         }
         
         if (isset($wbEv->rom))
         {      
            
            unset($wbEv->rom);
         }  
         
         $windows = array( $wbEv->metatileEditor, 
                           $wbEv->marioEditor, 
                           $wbEv->enemyEditor, 
                           $wbEv->colorEditor
                          );
                          
         foreach ($windows as $v)
         {  if ($v->isOpen())
            {  $v->closeWindow();
            }
         }         
               
         $rom = new SMBrom($filename);
         $wbEv->rom =& $rom;
         $em = $wbEv->em;
         
         $rom->seekTo(0x8010);
         $rom->bank0 = $rom->getChrrom();
         $rom->bank1 = $rom->getChrrom();      
   
         $wbEv->playerColors = $rom->getColors('PlayerColors', 12 , 1);      
         
         $wbEv->marioEditor->initWindow(
            $rom->bank0, $wbEv->playerColors[0], $rom->getMarioNames());
   
         $wbEv->colors = $rom->getColors('GroundAreaColors');                        
               
         $wbEv->metatileEditor->initWindow(
            $rom->bank1,  $wbEv->colors[0], $rom->getMetaTileNames());     
        
         $wbEv->enemyEditor->initWindow(
            $rom->bank0,  $wbEv->colors[5], $rom->getEnemyNames());
                                 
         foreach ($this->menuItem as $n=>$v)
         {  wb_set_enabled(wb_get_control($this->menu, $v), true);
         }
         
         $this->toConsole("$filename opened.\n", false);
      }
   }
   
   
   
   function saverom($params)
   {  $wbEv =& $params['wbEvents'];
      $rom =& $wbEv->rom;
      $filename = wb_sys_dlg_save($this->window, 'Save As...', $wbEv->file_filter);
      $rom->saveRom($filename);
      
      $this->toConsole("$filename saved.\n");
      
   }


   function toConsole($text, $doAppend = true)
   {  global $wbEvents;
      wb_set_text($wbEvents->console, ($doAppend ? wb_get_text($wbEvents->console) : '') . $text);
   }
   
   
   public function checkSize($params)
   {  
      
      $size = $this->getSize();
      if ($this->windowSize != $size )
      {  $params['param1'] = WBC_RESIZE;
         $params['param2'] = 'noEvent';
          
          $this->windowHandler($params);
          $this->windowSize = $size;          
      }
      return;
   }
   
   public function getSize($param = null)
   {  $ret = wb_get_size($this->window, $param);
      if (is_array($ret))
      {  $ret['width'] = $ret[0];
         $ret['height'] = $ret[1];
      }
      return $ret;
   }
   
   function sizeConsole()      
   {  
      $wbEv =& $this->wbEv;
      $size = $this->getSize();
      if ($this->console !== false)
      {
         wb_set_size($this->console, $size['width'] - 29, $size['height'] - 76 - (1 * $wbEv->em));
      }
   }
   
   function & getWindow()
   {  return $this->window;
   }
}


?>