<?php

class nesfile
{  
   private $filename;
   private $rom = false;
   private $loc;
   private $nesColorTable = array(
               0x00 => 0x787878, 0x01 => 0x2000b0, 0x02 => 0x2800b8, 0x03 => 0x6010a0, 
               0x04 => 0x982078, 0x05 => 0xb01030, 0x06 => 0xa03000, 0x07 => 0x784000, 
               0x08 => 0x485800, 0x09 => 0x386800, 0x0a => 0x386c00, 0x0b => 0x306040, 
               0x0c => 0x305080, 0x0d => 0x000000, 0x0e => 0x000000, 0x0f => 0x000000, 
               
               0x10 => 0xb0b0b0, 0x11 => 0x4060f8, 0x12 => 0x4040ff, 0x13 => 0x9040f0, 
               0x14 => 0xd840c0, 0x15 => 0xd84060, 0x16 => 0xe05000, 0x17 => 0xc07000, 
               0x18 => 0x888800, 0x19 => 0x50a000, 0x1a => 0x48a810, 0x1b => 0x48a068, 
               0x1c => 0x4090c0, 0x1d => 0x000000, 0x1e => 0x000000, 0x1f => 0x000000, 
               
               0x20 => 0xe8e8e8, 0x21 => 0x60a0ff, 0x22 => 0x5080ff, 0x23 => 0xa070ff, 
               0x24 => 0xf060ff, 0x25 => 0xff60b0, 0x26 => 0xff7830, 0x27 => 0xffa000, 
               0x28 => 0xe8d020, 0x29 => 0x98e800, 0x2a => 0x70f040, 0x2b => 0x70e090, 
               0x2c => 0x60d0e0, 0x2d => 0x787878, 0x2e => 0x000000, 0x2f => 0x000000, 
               
               0x30 => 0xffffff, 0x31 => 0x90d0ff, 0x32 => 0xa0b8ff, 0x33 => 0xc0b0ff, 
               0x34 => 0xe0b0ff, 0x35 => 0xffb8e8, 0x36 => 0xffc8b8, 0x37 => 0xffd8a0, 
               0x38 => 0xfff090, 0x39 => 0xc8f080, 0x3a => 0xa0f0a0, 0x3b => 0xa0ffc8, 
               0x3c => 0xa0fff0, 0x3d => 0xa0a0a0, 0x3e => 0x000000, 0x3f => 0x000000);
   
   function __construct($filename = false)
   {  //$this->filename = fopen($filename, 'rb');
     if ($filename !== false)
     { $this->rom = file_get_contents($filename);
     }
     
     $this->filename = $filename;
   }
   
   function openRom($filename)
   {  
      $this->rom = file_get_contents($filename);     
      $this->filename = $filename;
     
   }
   
   function saveRom($newfilename = false)
   {  if ($newfilename === false)
      {  $newfilename = $this->filename;
      }
      
      file_put_contents($newfilename, $this->rom);
   }
      
   function & getChrrom()
   {  $tobuild = array();
      for ($i = 0; $i <= 0x0f; $i++)
      {  for ($j = 0; $j <= 0x0f; $j++)
         {  $tobuild[$j][$i] = $this->getNextTile();
         }
      }
      return $tobuild;
   }      
   
   function seekTo($loc)
   {
      //fseek($this->filename, $loc);
      $this->loc = $loc;
        
   }
   
   function getBytes($len = 0)
   {  
      $ret = substr($this->rom,$this->loc,$len);
      $this->loc += $len;
      return $ret;
   } 
   
   function putBytes($bytes, $len = false)
   {  
      if ($len === false)
      {  $len = strlen($bytes);
      }   
      $this->rom = substr_replace($this->rom, $bytes, $this->loc, $len);
      $this->loc += $len;
      
   }
   
   function getNextTile()
   {  
      
      $ret = $this->processTile(substr($this->rom,$this->loc,0x10));
      $this->loc += 0x10;
      return $ret;
   }  

   
   function processTile($bytes)
   {  for ($i = 0; $i < 8; $i++)
      {  
         $output[$i] = $this->processRow((substr($bytes,$i,1)), (substr($bytes,$i+8,1)));     
      }
      
      return $output;
   }
   
   function processRow($byte1, $byte2)
   {  
      $byte1 = ord($byte1);
      $byte2 = ord($byte2);
      for ($i = 7; $i >= 0; $i--)
      {  $output[$i] = ($byte1 & 0x01);
         $byte1 = (int)$byte1 >> 1;
         
         $output[$i] = $output[$i] + ( 2 * ( $byte2 & 0x01 ) );
         $byte2 = (int)$byte2 >> 1;
      }
   
   
      return $output;
   }
   
   function getTileImage($bankimg, $loc = 0x00, $scale = 1)
   {
      $x = ($loc & 0xf0) / 0x10;
      $y = ($loc & 0x0f);
      $scale = 8 * $scale;
      return FreeImage_Copy($bankimg, $y * $scale, $x * $scale,($y + 1) * $scale, ($x + 1) * $scale);
   }
   
   function createTileArray($x, $y, $bankimg, $tilelistarr, $locs)
   {
      $output = Freeimage_Allocate($x * 8,$y * 8,24);
      $max = $x * $y;
      for($i = 0; $i < $max; $i++)
      {  $img[$i] = $this->getTileImage($bankimg,$tilelistarr[$i]);
         Freeimage_Paste($output, $img[$i],$locs[$i*2] * 8,$locs[($i*2)+1] * 8, 256);
         FreeImage_Unload($img[$i]);
      }
      
      return $output;
   }
   
   function buildBank($bank, $colors)
   {  $bankbits = $this->buildRawBits($colors, $bank);
      return FreeImage_ConvertFromRawBits($bankbits, 128, 128, 128*3, 24);
   }
   
   function buildRawBits($rgbmap, $tobuild = false)
   {  if ($tobuild === false)
      {  $tobuild = $this->getChrrom();
      }  
      
      // if still failed, then exit
      if ($tobuild === false) 
      {  return false;
      }  
      
      foreach ($rgbmap as $n=>$v)
      {  $rgbmap[$n] = $this->colorBytestring($this->nesColorTable[$v]);
      }
      $output = '';
      for ($table_row = 0; $table_row < 0x10; $table_row++)
      {  for ($tile_row = 0; $tile_row < 0x08; $tile_row++)      
         {  for ($table_col = 0; $table_col < 0x10; $table_col++)
            {  for ($tile_col = 0; $tile_col < 0x08; $tile_col++)
               {                    
                  $output .= 
                     $rgbmap[$tobuild[$table_col][$table_row][$tile_row][$tile_col]                                                ];
               }
            }   
         }
      }
      
      return $output;
   }


   function colorByteString($val, $bytenum = 3)
   {  $ret = '';
         
      for ($bytenum ; $bytenum > 0; $bytenum--)
      {  $exp = 8 * ($bytenum - 1);  
         $ret = chr($val / pow(0x100 , ($bytenum - 1)) ) . $ret;
         $val = $val - (($val >> $exp) << $exp);      
      }
   
      return $ret;
   }
   
   function nesColor($color)
   {  if ($color < 0x3f)
      {  return ($this->nesColorTable[$color]);
      }
      else
      {  return 0x0f;
      }
     
   }
   
   function buildNESPalette($colorList = false, $x = 16, $y = 4)
   {
      if ($colorList === false)
      {  for ($i = 0; $i <= 0x3f; $colorList[$i] = $i, $i++);
      }
      $pal = '';
      for ($i = 0; $i < ($x * $y); $i++)
      {  $pal .= $this->colorByteString($this->nesColor($colorList[$i]));
      }
      
      return FreeImage_ConvertFromRawBits($pal, $x, $y, $x * 3, 24);
   }
   // end nesfile
}




?>