<?php

   // Enemy Editor functions
class tileEditorWindow
{
   protected $wbEv;
   
   protected $windowTitle;
   protected $tileX, $tileY;
   protected $bankimg = false;
   protected $tileTable;
   protected $window;
   protected $selectedElement;
   protected $colorArr, $chrrom, $tileNames;
   protected $nameList, $numberBox, $tileFrame, $bankFrame;
   protected $currentColor = false;
   
   
   function __construct($windowTitle, $tileX, $tileY, $tileTableName, $tileLocArray)
   {  
      $this->wbEv =& $GLOBALS['wbEvents'];
      
      
      $this->tileX = $tileX;
      $this->tileY = $tileY;
      $this->tileTableName = $tileTableName;     
      $this->windowTitle = $windowTitle;   
      
      $this->tileLocArray = $tileLocArray;
   }

   function initWindow($bank, $colors, $tileNames)
   {
      $this->getTileTable();
      $this->setTileNames($tileNames);
      $this->setBankImage($bank, $colors);               
   }

   function getTileTable()
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;
      $memloc = $rom->getMemoryLoc($this->tileTableName);
      $rom->seekTo($memloc[0]);                    
      $this->tileTable = $rom->getBytes($memloc[1]);    
   }
   
   function createWindow($doDraw = false)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;   
      $em = $wbEv->em;
      
      $size = ($this->tileX * $this->tileY) * 2;
      $mainWindow =& $wbEv->mainWindow;
      $wbEv->createWindowEvent(array($this,'windowHandler'),
         $this->window = wb_create_window(  
            $mainWindow->getWindow(), ModelessDialog  , $this->windowTitle, 
               $this->window[0], $this->window[1], $em * $size + 45 + 256, $em * 10 + 55 +256, 
                  WBC_NOTIFY));
      
      $window =& $this->window;
      
      $this->nameList = wb_create_control(
         $window, ListBox, '', 10, 276,$em * $size + 20 + 256, $em*10, 
            $wbEv->createEvent(array($this,'listBoxEvent')));
     
      $this->numberBox = wb_create_control(
         $window, EditBox, '', 10, 16 + ($this->tileY * 32), $em * $size, $em  * 2, 
            $wbEv->createEvent(array($this,'numberBoxEvent')), 0, 0, 0);
     
      $this->frame = wb_create_control(
         $window, Frame, '', 10, 10, 32 * $this->tileX, 32 * $this->tileY, 
            ID, WBC_IMAGE);
   
      $this->bankFrame = wb_create_control(
         $window, Frame, "", $em * $size + 20 ,10, 256, 256, 
            ID, WBC_IMAGE);
            
      wb_set_text($this->nameList,$this->tileNames);
      wb_set_selected($this->nameList, 0); 
                 
      if ($doDraw)
      {  $this->drawBankImage();
      }
      
      
   }      

   function createTile($tileset, $tilelist)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;
      return $rom->createTileArray(
         $this->tileX, $this->tileY, $tileset,$tilelist, $this->tileLocArray);
   
   }

   function setTileNames($tilenames)
   {
     $this->tileNames = $tilenames; 
   }
   
   function setBankImage($chrrom, $colorArr)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;  
      
      $this->colorArr = $colorArr;            
      $this->chrrom = $chrrom;
   }
   
   function setBankColor($colorArr)
   {
      $this->colorArr = $colorArr;            
   }
   
   function drawBankImage()
   {  $rom =& $this->wbEv->rom;

      $bankbits = $rom->buildRawBits($this->colorArr, $this->chrrom);
      $this->bankimg = FreeImage_ConvertFromRawBits($bankbits, 128, 128, 128*3, 24);
      $bankimg_resized = Freeimage_Rescale($this->bankimg, 256, 256);
      drawFreeimage($this->bankFrame, $bankimg_resized);
      
   }
   
   function showEditor($params)
   {  
         
      $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;   
      $em = $wbEv->em;
      
      if (isset($this->window) &&  !is_array($this->window))
      {  wb_set_focus($this->window);
         return;
      }
      $mainWindow =& $wbEv->mainWindow;
      if (!is_array($this->window))
      {  $screenloc = wb_get_position($mainWindow->getWindow());
         if ($screenloc[0] == 65532) { $screenloc[0] = 1; }
         if ($screenloc[1] == 65532) { $screenloc[1] = 1; }
         
         
         $this->window[0] = $screenloc[0]+4*$em;
         $this->window[1] = $screenloc[1]+4*$em;
      }      
      
      
      $this->createWindow();
      $this->listBoxEvent($params);
      
   }

   
   
   function listBoxEvent($params, $notext = false)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;
      
      $this->selectedElement = wb_get_selected($this->nameList);
      
      $result = $this->processListBox($this->selectedElement, $this->tileTable, $this->tileX * $this->tileY);
      
      $tile = $this->createTile($this->bankimg, $result[0]);      
      drawFreeimage($this->frame, resizeFreeimage($tile, $this->tileX * 8 * 4, $this->tileY * 8 * 4));              
      
      if (!$notext)
      {  wb_set_text($this->numberBox,$result[1]);
      }
      
      
   }
   
   
   function numberBoxEvent($params)
   {    
      $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;
      
      $str = wb_get_text($this->numberBox);
      $str = str_replace(' ', '', $str);
      
      
      $bytelen = $this->tileX * $this->tileY;
      if (strlen($str) == $bytelen  * 2)
      {
         $tilearr = array();
         $bytes = '';
         for($i = 0;$i < $bytelen; $i++)
         {  
            $tilearr[$i] = hexdec(substr($str,$i*2,2));
            $bytes  .= chr($tilearr[$i]);
         }
        
         if ($this->selectedElement !== false)
         {  $this->tileTable = 
               substr_replace($this->tileTable, $bytes, $this->selectedElement * $bytelen, $bytelen);   
            
            $memloc = $rom->getMemoryLoc($this->tileTableName);
            $rom->seekTo($memloc[0]);  
            $rom->putBytes($this->tileTable);
            $this->listBoxEvent($params,true);
         }
      }
   }
   
   function windowHandler($params)
   {  if ($params['id'] == IDCLOSE)
      {  $this->closeWindow();
      } 
      
      return true;
   }
   
   function closeWindow()
   {     $this->unloadBankImg();
         $loc = wb_get_position($this->window);
         wb_destroy_window($this->window);
         $this->window = $loc;
         $this->currentColor = false;
   }
   
   function unloadBankImg()
   {
         if ($this->bankimg)
         {  Freeimage_Unload($this->bankimg);
            $this->bankimg = false;
         }
         
   }
   
   function processListBox($selectedEl, $graphicsTable, $size)
   {  
      
      
      $str = substr($graphicsTable, $selectedEl * $size, $size);
      
      $tilearr = array();
      $numberText = '';
      for($i = 0;$i < $size; $i++)
      {  
         $tilearr[$i] = ord(substr($str,$i,1));
         $numberText .= str_pad(dechex($tilearr[$i]),2,'0',STR_PAD_LEFT) . ' ';
      }      
      
      return array($tilearr, $numberText);
   }
   
   function isOpen()
   {
      return (isset($this->window) && !is_array($this->window));
   } 
   
   function updateColors()   
   {  
      
      if ($this->isOpen())
      {  $this->currentColor = false;
         $this->listBoxEvent(null, true);
      }
   }  
}

?>