<?php

class MarioEditorWindow extends tileEditorWindow
{  function __construct($title)
   {
      parent::__construct($title, 2, 4, 'MarioGraphicsTable', 
            array(0,0, 1,0, 0,1, 1,1, 0,2, 1,2, 0,3, 1,3));
            
   }
   
   function createWindow()
   {
      $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;   
      $em = $wbEv->em;
      $size = ($this->tileX * $this->tileY) * 2;
      parent::createWindow();
   
      $this->colorCombo = wb_create_control(
            $this->window, ComboBox, '', 10, 2*$em  + 32 * $this->tileY + 30,$em*$size, $em*10, 
               $wbEv->createEvent(array($this,'comboBoxEvent')), WBC_READONLY );
               
      wb_set_text($this->colorCombo, "Mario Color\nLuigi Color\nFire power color");               
      wb_set_selected($this->colorCombo, 0);
      $this->comboBoxEvent(); 
   }
   
   function comboBoxEvent($params = null)
   {  
      $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;   
      $em = $wbEv->em;
      $this->unloadBankImg();
      $this->setBankImage($rom->bank0, $wbEv->playerColors[wb_get_selected($this->colorCombo)]); 
      $this->drawBankImage();
      $this->listBoxEvent($params,true);
   }
   
   function updateColors()   
   {  
      if ($this->isOpen())
      {
         $this->comboBoxEvent();
      
      }
   }
   // end MarioEditorWindow class
}

class EnemyEditorWindow extends tileEditorWindow
{
   function __construct($title)
   {
      parent::__construct($title, 2, 3, 'EnemyGraphicsTable', 
            array(0,0, 1,0, 0,1, 1,1, 0,2, 1,2));
   }
   
   private $colorOffsets;
   
   
   function initWindow($bank, $colors, $tileNames)
   {  parent::initWindow($bank, $colors, $tileNames);
      
      $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom; 
      

      $EnemyGfxTableOffsets = $rom->getMemoryBlock('EnemyGfxTableOffsets');
      $EnemyAttributeData = $rom->getMemoryBlock('EnemyAttributeData');
      $this->colorOffsets = array();
      
      for($i = 0; $i < 27; $i++)      
      {  $loc = ord(substr($EnemyGfxTableOffsets, $i, 1)) / 0x06 ;
         $this->colorOffsets[$loc] = ord(substr($EnemyAttributeData,$i,1)) & 7;
         
         // sets up sprite coloring for extra sprite frames
         $offset = false;
         switch ($loc)
         {  case 0:
               if (!$offset) { $offset = 19; }
               // fall thru
           case 2:
               if (!$offset) { $offset = 15; }    
               
               $this->colorOffsets[$offset++] = $this->colorOffsets[$loc];
               $this->colorOffsets[$offset++] = $this->colorOffsets[$loc];
               $this->colorOffsets[$offset++] = $this->colorOffsets[$loc];
               $this->colorOffsets[$offset++] = $this->colorOffsets[$loc];
               $this->colorOffsets[$loc + 1] = $this->colorOffsets[$loc];
               break;            
            case 6:
            case 28:
            case 35:
               $this->colorOffsets[$loc + 1] = $this->colorOffsets[$loc];
               $this->colorOffsets[$loc + 2] = $this->colorOffsets[$loc];
               $this->colorOffsets[$loc + 3] = $this->colorOffsets[$loc];                        
               break;
            case 14:
               $this->colorOffsets[23] = $this->colorOffsets[$loc];
               break;
            default :            
               $this->colorOffsets[$loc + 1] = $this->colorOffsets[$loc];
         }   
      }      
   }

   
   function listBoxEvent($params, $notext = false)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;  
      
      $color = 0;
      $this->selectedElement = wb_get_selected($this->nameList);
      if ($this->selectedElement !== false)
      {  $color = $this->colorOffsets[$this->selectedElement];
         
         if ($color !== $this->currentColor)
         {
            $this->unloadBankImg();
            $this->setBankImage($rom->bank0, $wbEv->colors[$color+4]); 
            $this->drawBankImage();         
            $this->currentColor = $color;
         }
      }      
      parent::listBoxEvent($params, $notext);
      
   }
   // end EnemyEditorWindow class
}


class MetatileEditorWindow extends tileEditorWindow
{  
   private $colorOffsets;
  
   
   function __construct($title)
   {
      parent::__construct($title,  2, 2, 'MetatileGraphicsTable', 
            array(0,0, 0,1, 1,0, 1,1));
   }
   
   function initWindow($bank, $colors, $tileNames)
   {  parent::initWindow($bank, $colors, $tileNames);
      
      $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom; 
      
      $metaTileOffsets = $rom->getMemoryBlock('MetatileGraphicsOffsets');
      $this->colorOffsets = array();
      
      $this->colorOffsets[0] = ord(substr($metaTileOffsets,0,1)) +
         (0x100 * ord(substr($metaTileOffsets, 4,1)));
         
      for ($i = 1; $i < 4; $i++)
      {  $this->colorOffsets[$i] = (ord(substr($metaTileOffsets,$i,1)) +
            (0x100 * ord(substr($metaTileOffsets,$i + 4,1))) - $this->colorOffsets[0])/4;
      }
      $this->colorOffsets[0] = 0;
   }
   
   function listBoxEvent($params, $notext = false)
   {  $wbEv =& $this->wbEv;
      $rom =& $wbEv->rom;  
      
      $color = 0;
      $selectedElement = wb_get_selected($this->nameList);
      if ($selectedElement !== false)
      {  for ($i = 0; $i < 4; $i++)
         {  if ($selectedElement >= $this->colorOffsets[$i])
            {  $color = $i;
            }
         }
         
         if ($color !== $this->currentColor)
         {
            $this->unloadBankImg();
            $this->setBankImage($rom->bank1, $wbEv->colors[$color]); 
            $this->drawBankImage();         
            $this->currentColor = $color;
         }
      }      
      parent::listBoxEvent($params, $notext);
      
   }   
   // end MetatileEditorWindow class
}

?>